package service

import (
	"be.ems/src/modules/monitor/model"
	"be.ems/src/modules/monitor/repository"
)

// 实例化服务层 SysJobLog 结构体
var NewSysJobLog = &SysJobLog{
	sysJobLogRepository: repository.NewSysJobLogImpl,
}

// SysJobLog 调度任务日志 服务层处理
type SysJobLog struct {
	// 调度任务日志数据信息
	sysJobLogRepository repository.ISysJobLog
}

// SelectJobLogPage 分页查询调度任务日志集合
func (s *SysJobLog) SelectJobLogPage(query map[string]any) map[string]any {
	return s.sysJobLogRepository.SelectJobLogPage(query)
}

// SelectJobLogList 查询调度任务日志集合
func (s *SysJobLog) SelectJobLogList(sysJobLog model.SysJobLog) []model.SysJobLog {
	return s.sysJobLogRepository.SelectJobLogList(sysJobLog)
}

// SelectJobLogById 通过调度ID查询调度任务日志信息
func (s *SysJobLog) SelectJobLogById(jobLogId string) model.SysJobLog {
	return s.sysJobLogRepository.SelectJobLogById(jobLogId)
}

// DeleteJobLogByIds 批量删除调度任务日志信息
func (s *SysJobLog) DeleteJobLogByIds(jobLogIds []string) int64 {
	return s.sysJobLogRepository.DeleteJobLogByIds(jobLogIds)
}

// CleanJobLog 清空调度任务日志
func (s *SysJobLog) CleanJobLog() error {
	return s.sysJobLogRepository.CleanJobLog()
}
